import ode

import geometry

class Condition:
	def check(self, robot):
		pass
	
	def draw(self):
		pass
	
	@staticmethod
	def get(condition_xml):
		type = condition_xml.getAttribute("Type")
		if "False" == type:
			return FalseCondition()
		elif "AllOrdered" == type:
			return AllOrderedConditionGroup(condition_xml)
		elif "Collision" == type:
			return CollisionCondition(condition_xml)
		raise Exception("Unrecognized condition type %s" % type)
		
class FalseCondition(Condition):
	def check(self, robot):
		return False

class ConditionGroup(Condition):
	def __init__(self, condition_xml):
		self.conditions = []
		for sub_condition_xml in condition_xml.getElementsByTagName("Condition"):
			self.conditions.append(Condition.get(sub_condition_xml))

class AllOrderedConditionGroup(ConditionGroup):
	def check(self, robot):
		while 0 != len(self.conditions):
			if False == self.conditions[0].check(robot):
				return False
			else:
				self.conditions = self.conditions[1:]
		return True
	
	def draw(self):
		if 0 != len(self.conditions):
			self.conditions[0].draw()

class _Result:
	def __init__(self):
		self.res = False

class CollisionCondition(Condition):
	def __init__(self, condition_xml):
		self.geometries = []
		self.ode_space = ode.Space(type=1)
		
		for geometry_xml in condition_xml.getElementsByTagName("Geometry"):
			self.geometries.append(geometry.Geometry.get_geometry(self.ode_space, geometry_xml))
	
	def check(self, robot):
		result = _Result()
		def collision_callback(arg, geom1, geom2):
			contacts = ode.collide(geom1, geom2)
			if contacts > 0:
				arg.res = True
		ode.collide2(robot.ode_space, self.ode_space, result, collision_callback)
		return result.res
		
	def draw(self):
		for geom in self.geometries:
			geom.draw()
