import xml.dom.minidom

import ode
from OpenGL import GL,GLU
import pygame

import screen
import robot
import environment
import data
import camera
import font
import texture
import joint_control
import condition

STEP_TIME = 0.03

STATUS_RUNNING = 1
STATUS_HAS_WON = 2
STATUS_HAS_LOST = 3

class LevelScreen(screen.Screen):
	def __init__(self, level_name):
		self.simulation_time = 0
		self.last_step_time = 0
		self.time_to_complete = 0
		self.ode_world = ode.World()
		self.robot = None
		self.environment = None
		self.victory_condition = None
		self.failure_condition = None
		self.camera = camera.Camera()
		self.ode_contact_joints = ode.JointGroup()
		self.game_status = STATUS_RUNNING
		self.label = None
		self.end = False
		
		level_data_xmldoc = xml.dom.minidom.parseString(data.load("levels",level_name+".xml"))
		level_xml = level_data_xmldoc.getElementsByTagName("Level")[0]
		self.label = level_xml.getAttribute("Label")
		self.time_to_complete = eval(level_xml.getAttribute("TimeToComplete"))
		robot_type = level_xml.getAttribute("RobotType")
		self.robot = robot.Robot(data.load("robots",robot_type+".xml"), self.ode_world)
		env_type = level_xml.getAttribute("Environment")
		self.environment = environment.Environment(data.load("environments",env_type+".xml"))
		victory_condition_xml = level_xml.getElementsByTagName("VictoryCondition")[0]
		self.victory_condition = condition.Condition.get(victory_condition_xml)
		failure_condition_xml = level_xml.getElementsByTagName("FailureCondition")[0]
		self.failure_condition = condition.Condition.get(failure_condition_xml)
		self.ode_world.setGravity((0,0,-9.8))
	
	def restart(self):
		self.game_status = STATUS_RUNNING
		self.simulation_time = 0
		self.last_step_time = 0
		self.robot.restart()
		self.environment.restart()
	
	def handle_event(self, event):
		if event.type == pygame.MOUSEMOTION:
			if event.buttons[2]:
				self.camera.move(event.rel)
		
		if event.type == pygame.MOUSEBUTTONDOWN:
			if event.button==1:
				if event.pos[0]>633 and event.pos[1]>539:
					self.restart()
					return
				self.select_joint(event.pos)
				
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_SPACE and self.game_status != STATUS_RUNNING:
				if self.game_status == STATUS_HAS_LOST:
					self.restart()
				else:
					self.end = True
				return
			if self.robot.selected_joint != None:
				selected_joint = self.robot.joints[self.robot.selected_joint]
				selected_joint.joint_control_string.process_key(event.key, event.unicode)
	
	def do_frame(self, time_delta):
		if self.game_status != STATUS_RUNNING:
			return self.end
		
		self.simulation_time += time_delta
		while self.simulation_time > self.last_step_time + STEP_TIME:
			self.step()
			self.last_step_time += STEP_TIME
		if self.game_status == STATUS_RUNNING:
			if self.victory_condition.check(self.robot):
				self.game_status = STATUS_HAS_WON
				return
			if self.failure_condition.check(self.robot):
				self.game_status = STATUS_HAS_LOST
				return
			if self.time_to_complete != 0 and self.time_to_complete <= self.simulation_time:
				self.game_status = STATUS_HAS_LOST
				return
	
	def step(self):
		for joint in self.robot.joints:
			joint.set_params(self.simulation_time)
		self.robot.ode_space.collide(None, self._collide_near_callback)
		ode.collide2(self.robot.ode_space, self.environment.ode_space, None, self._collide_near_callback)
		self.ode_world.step(STEP_TIME)
		self.ode_contact_joints.empty()
	
	def _collide_near_callback(self, arg, geom1, geom2):
		contacts = ode.collide(geom1, geom2)
		for contact in contacts:
			contact.setMu(1)
			contact_joint = ode.ContactJoint(self.ode_world, self.ode_contact_joints, contact)
			contact_joint.attach(geom1.getBody(), geom2.getBody())
	
	def draw(self):
		GL.glDepthMask(1)
		GL.glClearColor(0.7,0.7,0.7,0)
		GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
		
		# 3D
		self.camera.init_3d()
		
		GL.glLoadIdentity()
		self.camera.apply((0,0,0))
		GL.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, (1,1,2,0))
		GL.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, (-1,-1,-2,0))
		self.robot.draw()
		self.environment.draw()
		
		self.victory_condition.draw()
		self.failure_condition.draw()
		
		# 2D
		self.camera.init_2d()
		
		GL.glScalef(2,-2,1)
		GL.glTranslatef(-0.5,-0.5,0)
		GL.glScalef(1.0/800,1.0/600,1)
		
		font.Font.get_font().draw("Time: %0.2f" % self.simulation_time)
		
		GL.glPushMatrix()
		GL.glTranslatef(10,570,0)
		font.Font.get_font().draw(self.label)
		GL.glPopMatrix()
		
		GL.glPushMatrix()
		GL.glTranslatef(633,539,0)
		texture.get("compile").draw()
		GL.glPopMatrix()
		
		if self.robot.selected_joint != None:
			selected_joint = self.robot.joints[self.robot.selected_joint]
			GL.glPushMatrix()
			GL.glTranslatef(10,400,0)
			texture.get("joint_menu").draw()
			GL.glTranslatef(30,22,0)
			font.Font.get_font().draw(selected_joint.label)
			GL.glTranslatef(0,34,0)
			selected_joint.joint_control_string.draw()
			GL.glPopMatrix()
		
		GL.glPushMatrix()
		GL.glTranslatef(208,182,0)
		if self.game_status == STATUS_HAS_LOST:
			texture.get("lose").draw()
		elif self.game_status == STATUS_HAS_WON:
			texture.get("win").draw()
		GL.glPopMatrix()
	
	def close(self):
		del self.ode_world
		self.robot.close()
		del self.robot
		del self.environment
		del self.victory_condition
		del self.failure_condition
		del self.camera
		del self.ode_contact_joints
		
	def select_joint(self, pos):
		#create selection matrices
		GL.glMatrixMode(GL.GL_PROJECTION)
		GL.glPushMatrix()
		GL.glLoadIdentity()
		viewport = GL.glGetIntegerv(GL.GL_VIEWPORT)
		GLU.gluPickMatrix(pos[0],600-pos[1],1,1,viewport)
		GLU.gluPerspective(45, 1.0, 1, 2000)
		
		GL.glMatrixMode(GL.GL_MODELVIEW)
		GL.glPushMatrix();
		GL.glLoadIdentity();
		self.camera.apply((0,0,0))

		#select
		GL.glInitNames();
		GL.glSelectBuffer(1024);
		GL.glRenderMode(GL.GL_SELECT);

		GL.glPushName(-1)
		for i in xrange(len(self.robot.joints)):
			GL.glLoadName(i)
			self.robot.joints[i].draw()
		GL.glPopName()

		#restore the matrices
		GL.glMatrixMode(GL.GL_PROJECTION);
		GL.glPopMatrix();
		GL.glMatrixMode(GL.GL_MODELVIEW);
		GL.glPopMatrix();

		GL.glFlush();

		selection_records=GL.glRenderMode(GL.GL_RENDER)
		nearest = None
		for selection_record in selection_records:
			if (None == nearest) or (selection_record.near < nearest.near):
				nearest = selection_record
		if nearest:
			self.robot.selected_joint = nearest.names[0]
		else:
			self.robot.selected_joint = None
		
